<?php

namespace Waylet\Payment\Controller\Onepage;

use Magento\Framework\App\Action\HttpGetActionInterface as HttpGetActionInterface;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Exception\NoSuchEntityException;
use Psr\Log\LoggerInterface;

/**
 * Onepage checkout success controller class
 */
class Success extends \Magento\Framework\App\Action\Action
{

    private $wayletLogger;
    private $checkoutSession;
    private $orderFactory;
    private $orderRepository;

    /**
     * Callback constructor.
     *
     * @param OrderFactory $orderFactory
     * @param OrderRepositoryInterface $orderRepository
     *
     */
    public function __construct(
        LoggerInterface $wayletLogger,
        \Magento\Checkout\Model\Session $session,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        Context $context
    ) {
        $this->wayletLogger = $wayletLogger;
        $this->checkoutSession = $session;
        $this->orderFactory = $orderFactory;
        $this->orderRepository = $orderRepository;
        parent::__construct($context);
    }

    /**
     * Order success action
     *
     * @return \Magento\Framework\App\ResponseInterface
     */
    public function execute()
    {
        $requestData = $this->getRequest()->getParams();
        $quoteId = $this->getQuoteIdFromRequest($requestData);

        try {
            if (!$quoteId) {
                throw new NoSuchEntityException();
            }

            $order = $this->getOrderByQuoteId($quoteId);
        } catch (NoSuchEntityException $e) {
            $this->wayletLogger->error($e->getMessage());
            return $this->redirectCart();
        }

        $this->checkoutSession->setLastQuoteId($quoteId);
        $this->checkoutSession->setLastSuccessQuoteId($quoteId);
        $this->checkoutSession->setLastOrderId($order->getId());
        $this->checkoutSession->getLastRealOrder($order);
        $this->checkoutSession->setLastRealOrderId($order->getIncrementId());
        $this->checkoutSession->setLastOrderStatus($order->getStatus());

        return $this->redirectSuccessPage();
    }

    private function redirectCart()
    {
        return $this->_redirect('checkout/cart');
    }

    private function getOrderByQuoteId($quoteId)
    {
        $orderModel = $this->orderFactory->create();
        $lastOrder = $orderModel->load($quoteId, 'quote_id');
        return $this->orderRepository->get($lastOrder->getId());
    }

    private function getQuoteIdFromRequest(array $requestData)
    {
        $quoteId = $requestData['externalId'];

        // El ID puede venir de la forma "{ID}-{timestamp}"
        if (strpos($quoteId, '-') !== false) {
            $parts = explode('-', $quoteId);
            if (is_array($parts) && isset($parts[0])) {
                $quoteId = $parts[0];
            }
        }

        $this->wayletLogger->info("Incoming quote Id: {$quoteId}");

        return $quoteId;
    }

    public function redirectSuccessPage(): \Magento\Framework\App\ResponseInterface
    {
        return $this->_redirect('checkout/onepage/success');
    }
}
