<?php

namespace Waylet\Payment\Controller\Payment;

use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\CsrfAwareActionInterface;
use Psr\Log\LoggerInterface;
use Magento\Framework\Webapi\Exception as WebapiException;
use Magento\Framework\Controller\Result\RedirectFactory;
use Magento\Checkout\Helper\Cart as CartHelper;
use \Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface as TBuilder;

class Process extends \Magento\Framework\App\Action\Action implements HttpPostActionInterface, CsrfAwareActionInterface
{
    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var Magento\Quote\Api\CartManagementInterface
     */
    private $cartManagementInterface;

    /**
     * @var Magento\Checkout\Model\Session
     */
    protected $checkoutSession;

    /**
     * @var Magento\Sales\Api\OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var Magento\Framework\AuthorizationInterface
     */
    protected $authorization;

    /**
     * @var Waylet\Payment\Helper\Data
     */
    protected $helperData;

    /**
     * @var Waylet\Payment\Helper\Api
     */
    protected $helperApi;

    /**
     * @var Magento\Framework\Encryption\EncryptorInterface
     */
    protected $encryptor;

    protected $order;

    private $wayletLogger;

    private $_transactionBuilder;

    /**
     * Callback constructor.
     *
     * @param Context $context
     * @param LoggerInterface $logger
     * @param Session $checkoutSession
     * @param OrderFactory $orderFactory
     * @param OrderRepositoryInterface $orderRepository
     * @param JsonFactory $resultJsonFactory
     * @param AuthorizationInterface $authorization
     * @param Data $helperData
     * @param Waylet\Payment\Helper\Api  $helperData
     *
     */
    public function __construct(
        Context $context,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Quote\Api\CartManagementInterface $cartManagementInterface,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Framework\AuthorizationInterface $authorization,
        \Waylet\Payment\Helper\Data $helperData,
        \Waylet\Payment\Helper\Api $helperApi,
        \Magento\Framework\Encryption\EncryptorInterface $encryptor,
        TBuilder $transactionBuilder
    ) {
        parent::__construct($context);
        $this->logger = $logger;
        $this->checkoutSession = $checkoutSession;
        $this->cartManagementInterface = $cartManagementInterface;
        $this->orderRepository = $orderRepository;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->authorization = $authorization;
        $this->helperData = $helperData;
        $this->helperApi = $helperApi;
        $this->encryptor = $encryptor;
        $this->_transactionBuilder = $transactionBuilder;
    }

    /**
     * @inheritDoc
     */
    public function execute()
    {
        // Debug
        $writer = new \Zend_Log_Writer_Stream(BP . '/var/log/waylet.log');
        $logger = new \Zend_Log();
        $logger->addWriter($writer);
        $this->wayletLogger = $logger;

        // Get request data
        $requestBody = $this->getRequest()->getContent();
        $requestData = json_decode($requestBody, true);

        //Process Paid
        if ($requestData['status'] == 'PAID') {
            $this->createOrderFromWayletResponse($requestData);
            // New Logic.
            // Create Invoice.
            $this->PAID($requestData);
            $this->createTransaction($this->order, $requestData);
        }

        //Process Refused
        if ($requestData['status'] == 'REFUSED') {
            $this->REFUSED($requestData);
        }

        $this->wayletLogger->info('Waylet:' . $requestBody);

        // Respond to API external with confirmation message
        $response = ['message' => 'Payment response processed'];
        $resultJson = $this->resultJsonFactory->create();
        $resultJson->setData($response);
        return $resultJson;
    }

    public function captureOperation($order = null, $paymentData = array())
    {
        $payment = $order->getPayment();
        $payment->setLastTransId($paymentData['wayletPaymentId']);
        $payment->setTransactionId($paymentData['wayletPaymentId']);
        $payment->setAdditionalInformation(
            [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData]
        );
        $payment->setIsTransactionPending(false);

        //$formatedPrice = $order->getBaseCurrency()->formatTxt($order->getGrandTotal());

        //$message = __('The authorized amount is %1.', $formatedPrice);

        $payment->capture();
    }

    public function createTransaction($order = null, $paymentData = array())
    {
        try {
            //get payment object from order object
            $payment = $order->getPayment();
            $payment->setLastTransId($paymentData['wayletPaymentId']);
            $payment->setTransactionId($paymentData['wayletPaymentId']);
            $payment->setAdditionalInformation(
                [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData]
            );
            $formatedPrice = $order->getBaseCurrency()->formatTxt($order->getGrandTotal());

            $message = __('The authorized amount is %1.', $formatedPrice);
            //get the object of builder class
            $trans = $this->_transactionBuilder;
            $transaction = $trans->setPayment($payment)
                                 ->setOrder($order)
                                 ->setTransactionId($paymentData['wayletPaymentId'])
                                 ->setAdditionalInformation(
                                    [\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $paymentData]
                                 )
                                 ->setFailSafe(true)
                                 ->setSalesDocument($payment->getCreatedInvoice())

                                //build method creates the transaction and returns the object
                                ->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_CAPTURE);

            $payment->addTransactionCommentsToOrder(
                $transaction,
                $message
            );
            $payment->setParentTransactionId(null);
            $payment->save();
            $order->save();

            return  $transaction->save()->getTransactionId();
        } catch (\Exception $e) {
            $this->wayletLogger->info("Error saving the transaction: {$e->getMessage()}");
        }
    }

    private function createOrderFromWayletResponse(array $requestData)
    {
        if (!isset($requestData['paymentId'])) {
            throw new \Exception('Quote ID not present in Waylet Response');
        }

        $quoteId = $this->getQuoteIdFromRequest($requestData);
        $orderId = $this->cartManagementInterface->placeOrder($quoteId);
        $order = $this->orderRepository->get($orderId);
        $this->wayletLogger->info("Order found with id: {$order->getId()}");

        $this->order = $order;
    }

    private function getQuoteIdFromRequest(array $requestData)
    {
        //$quoteId = $requestData['paymentId'];
        list($quoteId, $time) = explode('-', $requestData['paymentId']);

        if (!$quoteId) {
            throw new \Exception('Quote ID is not exist.');
        }
        /*
        if (time() - $time > 3600) {
            throw new \Exception('Quote ID is expired.');
        }
        */

        $this->wayletLogger->info("Incoming quote Id: {$quoteId}");

        return $quoteId;
    }

    /**
     * Method to process paid and change order status to process
     */
    public function PAID($requestData)
    {
        $order = $this->order;
        $status = \Magento\Sales\Model\Order::STATE_PROCESSING;

        // Create Invoice and Transaction.
        $this->captureOperation($order, $requestData);

        // Actualizar el estado de la orden
        $order->setState($status);

        // Actualizar el estado del pago
        $order->setStatus($order->getConfig()->getStateDefaultStatus($status));
        $order->addCommentToStatusHistory(__('Payment accepted received from Waylet Gateway'), $status);
        $order->save();

        $logMessage = "Payment ID: " . $requestData['paymentId'] . " has been updated to PAID status.";
        $this->wayletLogger->info($logMessage);
    }

    /**
     * Method to process paid and change order status to cancel
     */
    public function REFUSED($requestData)
    {
        $this->wayletLogger->info(__("Payment " . $requestData['paymentId'] . " refused received from Waylet Gateway"));
    }

    public function validateForCsrf(RequestInterface $request): ?bool
    {
        $logMessage = "New incoming process payment";
        $this->logger->log(\Psr\Log\LogLevel::INFO, $logMessage);

        // Get request data
        $requestBody = $this->getRequest()->getContent();
        $requestData = json_decode($requestBody, true);

        return $this->helperApi->verifyHas($requestData, $this->encryptor->decrypt($this->helperData->getSecret()));
    }

    public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException
    {
        throw new WebapiException(__('Invalid CSRF token.'));
    }
}
