<?php

namespace Waylet\Payment\Controller\Payment;

use Magento\Framework\App\Action\Context;
use Psr\Log\LoggerInterface;

class Success extends \Magento\Framework\App\Action\Action
{
    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var Magento\Checkout\Model\Session 
     */
    protected $checkoutSession;

    /**
     * @var Magento\Sales\Model\OrderFactory
     */
    protected $orderFactory;

    /**
     * @var Magento\Sales\Api\OrderRepositoryInterface
     */
    protected $orderRepository;

    protected $order;
    private $wayletLogger;

    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    protected $resultPageFactory;

    /**
     * Callback constructor.
     *
     * @param Context $context
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Sales\Model\OrderFactory $orderFactory
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     */
    public function __construct(
        Context $context,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory
    ) {
        parent::__construct($context);
        $this->logger = $logger;
        $this->checkoutSession = $checkoutSession;
        $this->orderFactory = $orderFactory;
        $this->orderRepository = $orderRepository;
        $this->resultPageFactory = $resultPageFactory;
    }

    /**
     * @inheritDoc
     */
    public function execute()
    {
        // Debug
        $writer = new \Zend_Log_Writer_Stream(BP . '/var/log/custom.log');
        $logger = new \Zend_Log();
        $logger->addWriter($writer);
        $this->wayletLogger = $logger;

        // Get request data
        $requestData = $this->getRequest()->getParams();
        $quoteId = $this->getQuoteIdFromRequest($requestData);

        $this->wayletLogger->info(print_r($requestData, true));

        $this->checkoutSession->clearQuote();
        $this->checkoutSession->clearStorage();
        $this->checkoutSession->restoreQuote();

        $orderModel = $this->orderFactory->create();
        $lastOrder = $orderModel->load($quoteId, 'quote_id');
        $order = $this->orderRepository->get($lastOrder->getId());

        $resultPage = $this->resultPageFactory->create();
        /*$this->_eventManager->dispatch(
            'checkout_onepage_controller_success_action',
            [
                'order_ids' => [$order->getId()],
                'order' => $order->getIncrementId()
            ]
        );*/

        return $resultPage;
    }

    private function getQuoteIdFromRequest(array $requestData)
    {
        $quoteId = $requestData['externalId'];

        // El ID puede venir de la forma "{ID}-{timestamp}"
        if (strpos($quoteId, '-') !== false) {
            $parts = explode('-', $quoteId);
            if (is_array($parts) && isset($parts[0])) {
                $quoteId = $parts[0];
            }
        }

        return $quoteId;
    }
}
