<?php

namespace Waylet\Payment\Controller\Payment;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\Result\Redirect;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Checkout\Model\Session as CheckoutSession;
use Waylet\Payment\Helper\Api;
use Waylet\Payment\Helper\Data;
use Magento\Framework\App\ResponseFactory;
use Magento\Framework\UrlInterface;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Quote\Model\Quote;

class WRedirect extends Action
{
    /**
     * @var OrderRepositoryInterface
     */
    protected $_orderRepository;

    /**
     * @var JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var Session
     */
    protected $checkoutSession;

    /**
     * @var \Waylet\Payment\Helper\Api
     */
    protected $helperApi;

    /**
     * @var \Waylet\Payment\Helper\Data
     */
    protected $helperData;

    /**
     * @var \Magento\Framework\App\ResponseFactory
     */
    protected $responseFactory;

    /**
     * @var \Magento\Framework\UrlInterface
     */
    protected $url;

    /**
     * @var \Magento\Framework\Encryption\EncryptorInterface 
     */
    protected $encryptor;
    protected $quote;
    protected $cartRepository;

    /**
     * Constructor
     *
     * @param Context $context
     * @param OrderRepositoryInterface $orderRepository
     * @param JsonFactory $resultJsonFactory
     * @param CheckoutSession $checkoutSession
     * @param Api $helperApi
     * @param Data $helperData
     * @param ResponseFactory $responseFactory
     * @param UrlInterface $url
     * @param EncryptorInterface $encryptor
     * @param Quote $quote
     * @param CartRepositoryInterface $cartRepository
     */
    public function __construct(
        Context $context,
        OrderRepositoryInterface $orderRepository,
        JsonFactory $resultJsonFactory,
        CheckoutSession $checkoutSession,
        Api $helperApi,
        Data $helperData,
        ResponseFactory $responseFactory,
        UrlInterface $url,
        EncryptorInterface $encryptor,
        Quote $quote,
        CartRepositoryInterface $cartRepository
    ) {
        $this->_orderRepository = $orderRepository;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->checkoutSession = $checkoutSession;
        $this->helperApi = $helperApi;
        $this->helperData = $helperData;
        $this->responseFactory = $responseFactory;
        $this->url = $url;
        $this->encryptor = $encryptor;
        $this->quote = $quote;
        $this->cartRepository = $cartRepository;
        parent::__construct($context);
    }

    /**
     * Execute view action
     *
     * @return ResponseInterface|Redirect
     */
    public function execute()
    {

        $response = new \stdClass();
        $quoteId = $this->checkoutSession->getQuote()->getId();
        $getGrandTotal =  $this->checkoutSession->getQuote()->getGrandTotal();

        $data = [
            "externalId" => $quoteId . '-' . time(),
            "commerceId" => $this->helperData->getCommerceId(),
            "amount" => (string)$getGrandTotal * 100,
            "callbackUrl" => $this->helperData->getCallbackUrl(),
            "urlOk" => $this->helperData->getUrlOk() . '?externalId=' . $quoteId . '-' . time(),
            "urlKo" => $this->helperData->getUrlKo()
        ];

        $responseApi = $this->helperApi->requestPaymentUrl($data, $this->helperData->isDebug(), $this->encryptor->decrypt($this->helperData->getApiKey()));

        $responseData = json_decode($responseApi);

        if (!empty($responseData->error)) {

            $response->success = false;
            $response->message = 'Error al procesar la solicitud: ' . $responseData->message;
        }
        if (!empty($responseData->url)) {

            $response->success = true;
            $response->data = $responseData->url;
        }

        $resultJson = $this->resultJsonFactory->create();
        $resultJson->setData($response);
        return $resultJson;
    }
}
