<?php

namespace Waylet\Payment\Model;

use Magento\Directory\Helper\Data as DirectoryHelper;
use Magento\Payment\Model\InfoInterface;
use Magento\Payment\Model\Method\AbstractMethod;

class Payment extends AbstractMethod
{
    /**
     * Determines of payment method code
     */
    const PAYMENT_METHOD_CODE = 'waylet_payment';

    protected $_code = self::PAYMENT_METHOD_CODE;

    /**
     * Determines of payment method is Authorize
     */
    protected $_canAuthorize = true;

    protected $_canCapture = true;

    protected $_canRefund = true;

    private $api;

    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
        \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
        \Magento\Payment\Helper\Data $paymentData,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Payment\Model\Method\Logger $logger,
        \Waylet\Payment\Helper\Api $api,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = [],
        DirectoryHelper $directory = null
    )
    {
        parent::__construct($context, $registry, $extensionFactory, $customAttributeFactory, $paymentData, $scopeConfig, $logger, $resource, $resourceCollection, $data, $directory);
        $this->api = $api;
    }


    /**
     * Determines if the method is offline
     */
    protected $_isOffline = false;

    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        return parent::isAvailable($quote) && !empty($this->getConfigData('title'));
    }

    /**
     * Authorize payment method
     *
     * @param \Magento\Framework\DataObject|InfoInterface $payment
     * @param float $amount
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @deprecated 100.2.0
     */
    public function authorize(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        if (!$this->canAuthorize()) {
            throw new \Magento\Framework\Exception\LocalizedException(__('The authorize action is not available.'));
        }

        // Mark the order as pending Gateway confirmation
        $payment->setIsTransactionClosed(false);
        $payment->setIsTransactionPending(true);

        return $this;
    }

    public function refund(InfoInterface $payment, $amount)
    {
        parent::refund($payment, $amount);

        $additionalInformation = $payment->getAdditionalInformation();
        $this->api->refundPayment([
            'paymentId' => $additionalInformation['raw_details_info']['paymentId'],
                            //$payment->getParentTransactionId(),
            'amount'    => $amount
        ]);

        return $this;
    }
}
