/*browser:true*/
/*global define*/
define(
    [
        'jquery',
        'Magento_Checkout/js/view/payment/default',
        'Magento_Customer/js/customer-data',
        'Magento_Checkout/js/action/set-payment-information',
        'Magento_Checkout/js/model/full-screen-loader',
        'mage/url',
		//'Magento_Ui/js/model/messageList',
    ],
    function ($, Component, customerData, setPaymentInformation, fullScreenLoader, url) {
        'use strict';

      	//var messageContainer = messageList;

        return Component.extend({
            defaults: {
                template: 'Waylet_Payment/payment/waylet_payment',
                redirectUrl: ''
            },

            clearCheckoutData: function () {
                const clearData = {
                    'selectedShippingAddress': null,
                    'shippingAddressFromData': null,
                    'newCustomerShippingAddress': null,
                    'selectedShippingRate': null,
                    'selectedPaymentMethod': null,
                    'selectedBillingAddress': null,
                    'billingAddressFromData': null,
                    'newCustomerBillingAddress': null
                };

                customerData.set('checkout-data', clearData);
                customerData.invalidate(['cart']);
            },

            /**
             * After place order redirect to waylet
             */
            afterPlaceOrder: function () {
                this.clearCheckoutData();
                $.mage.redirect(this.redirectUrl);
                return;
            },

            /**
             * Get payment method data
             * @returns {{method: *}}
             */
            getData: function () {
                return {
                    'method': this.item.method,
                	'po_number': null,
                    'additional_data' : null
                };
            },

            /**
             * Place order.
             */
            placeOrder: function () {
                var self = this;

                if (this.validate()) {
                    this.isPlaceOrderActionAllowed(false);

                    if (this.validatePaymentMethod(fullScreenLoader)) {
                        fullScreenLoader.startLoader();
                        $.when(
                          	//placeOrderAction(this.getData(), this.redirectAfterPlaceOrder, this.messageContainer)
                            setPaymentInformation(this.messageContainer, this.getData())
                        ).done(function () {
                            self.afterPlaceOrder();
                        }).fail(function () {
                            fullScreenLoader.stopLoader();
                            self.isPlaceOrderActionAllowed(true);
                        });
                    }
                }
            },

            /**
             * Validate payment method
             * @returns {boolean}
             */
            validatePaymentMethod: function (loader) {

                var self = this;
                var bandera = false;
                var wredirect = url.build('waylet_payment/payment/wredirect');
                loader.startLoader();

                $.ajax({
                    url: wredirect,
                    type: "POST",
                    dataType: 'json',
                    global: false,
                    async: false,
                    success: function (response) {
                        loader.stopLoader();
                        if (response.success) {
                            self.redirectUrl = response.data;
                            bandera = true;
                        } else {

                            $('.waylet-message').remove();
                            $('#waylet-payment-error-message').before(
                                '<div role="alert" class="waylet-message message message-error error">' +
                                '<div class="scoring-failed">' + $.mage.__("An error occurred while trying to get the payment url, please try again.") + '</div>' +
                                '</div>'
                            );
                        }
                    },
                    error: function (response) {
                        loader.stopLoader();
                        $('.waylet-message').remove();
                        $('#waylet-payment-error-message').before(
                            '<div role="alert" class="waylet-message message message-error error">' +
                            '<div class="scoring-failed">' + $.mage.__("An error occurred while trying to get the payment url, please try again.") + '</div>' +
                            '</div>'
                        );
                    }
                });
                return bandera;
            },

            /**
            * Get image url.
            */
            getLogoUrl: function () {
                return window.checkoutConfig.payment.waylet_payment.logoUrl
            },

            /**
             * Validate if payment method is active
             * @returns {boolean}
             */
            isActive: function () {
                return this.item.method === 'waylet_payment';
            }
        });
    }
);
